<?php
/***********************************************************************************
Script purpose: Calculating numbers for a 3n-1 sequence (using the GMP libary)
Written by:		Joerg Drescher, iovialis@gmx.de
Date: 			31.03.2022
Link: 			https://3n-1.de/sequence_3n-1.php
Lizence: 		CC BY-SA - https://creativecommons.org/licenses/by-sa/4.0/
***********************************************************************************/
error_reporting (E_ALL ^ E_NOTICE); 					// Switch Error-Messages off

$bin          = strval($_POST["bin"]);					// Getting the selected binary number
$dec          = strval($_POST["dec"]);					// Getting the selected decimal number
$dec_or_bin   = strval($_POST["dec_or_bin"]);			// Getting the selection, what number is used (binary or decimal number)
$show_even	  = strval($_POST["show_even"]);			// Showing evens or only odds

if (!$bin) { $bin = "11011"; }							// Default binary number (11011 - decimal 27)
if (!$dec_or_bin)  { $dec_or_bin  = "true"; }			// Default usage bin
if (!$show_even)   { $show_even   = "true"; }			// Default show odds and evens

if ($dec_or_bin == "true") { 							// If usage is binary numbers
	$dec = gmp_init($bin,2); 							// calculate decimal number
} else {												// If usage is decimal number (else)
	$bin = gmp_strval($dec,2);							// calculate binary number
}
/*
HTML for the form
*/
?>

<html> 
  <head> 
    <meta charset="utf-8">
    <title>Sequence calculator 3n-1</title>
	<style>

      td, th { 
		border: 1px solid;
		font-family: Courier;
		font-size: 12px;
		}
	  tr:hover {
		  background-color: #ddd;
	  }


    </style>
	
  </head>
  <body>
  <h3>Sequence calculator 3n-1<h3>
<form name="Formular" action="" method="POST">
<table border="1">
	<tr>
		<td align="right" width="120">
			Start (bin):
		</td>
		<td>
			<input name="bin" type="text" size="100" maxlength="300" value="<?php echo $bin; ?>" >
		</td>
	</tr>
	<tr>
		<td align="right" width="120">
			Start (dec):
		</td>
		<td>
			<input name="dec" type="text" size="100" maxlength="300" value="<?php echo $dec; ?>" >
		</td>
	</tr>
	<tr>
		<td align="right" width="120">
			Usage:
		</td>
		<td>
			Bin <input type="radio" name="dec_or_bin" value="true"  <?php if ( $dec_or_bin == "true"  ) { echo "checked"; } ?> >
			Dec <input type="radio" name="dec_or_bin" value="false" <?php if ( $dec_or_bin == "false" ) { echo "checked"; } ?> >
		</td>
	</tr>
	<tr>
		<td align="right" width="120">
			Show even:
		</td>
		<td>
			Yes <input type="radio" name="show_even" value="true"  <?php if ( $show_even == "true"  ) { echo "checked"; } ?> >
			No <input type="radio" name="show_even" value="false"  <?php if ( $show_even == "false" ) { echo "checked"; } ?> >
		</td>
	</tr>
	<tr>
		<td>
			&nbsp;
		</td>
		<td>
			<input type="submit" value=" Send " />
			</form>
		</td>
	</tr>
</table>

<?php

$i=1;									// Set counter for iterations
// *************************************** Output of the first iteration
echo "<table border=\"1\">";
echo "<tr><th>Iteration</th><th>Decimal</th><th>Binary</th></tr>";
echo "<tr><td width='100'>".$i.".</td><td width='100'>".$dec."</td><td width='100' align='right'>".$bin."</td></tr>";

while($i <= 3000) {						// Execute iterations (with maximum 3000)
	$i++;								// Increase the iteration number
	$decmod = gmp_mod($dec,"2");			// n mod 2 - test, if n is odd or even
	if ($decmod == "0") {					// if n is even
		$bin = instr_even($bin);			// executing n / 2
	} else {								// if n is odd
		$bin = instr_odd($dec);				// executing n * 3 + 1
	}
	$dec = gmp_init($bin,2);				// generating the decimal number of the result
	$decmod = gmp_mod($dec,"2");	 		// test, if the the new result is even

	if ($show_even == "true" AND $decmod == "0") { 	// if the new result is even and showing even is selected, output of the result
		echo "<tr><td width='100'>".$i.".</td><td width='100'>".$dec."</td><td width='100' align='right'>".$bin."</td></tr>\n";
	} 
	if ($decmod == "1") {							// if the new result is odd, output of the result
		echo "<tr><td width='100'>".$i.".</td><td width='100'>".$dec."</td><td width='100' align='right'>".$bin."</td></tr>\n";
	}
	
	if ( $dec == "1" OR $dec == "5" OR $dec == "17" ) { // if the end of a loop is reached, stop
		break 1;
	}
}

echo "</table>";

function instr_even($bin) {						// executing the instruction for even numbers (using binary input)
	$bin = substr($bin, 0, -1);					// delete the last 0
	return $bin;
}

function instr_odd($dec) {						// executing the instruction for odd numbers (using decimal input)
	$dec    = gmp_sub(gmp_mul($dec,"3"),"1");	// n * 3 - 1
	$bin    = gmp_strval($dec,2);				// converting to a binary number
	return $bin;
}

?>